@@ -14,11 +14,13 @@
 #    under the License.
 
 import copy
-import httplib
 import logging
 import os
+import posixpath
 import socket
-import urlparse
+
+from heatclient.openstack.common.py3kcompat import urlutils
+from six.moves import http_client as httplib
 
 try:
     import ssl
@@ -31,11 +33,6 @@ try:
 except ImportError:
     import simplejson as json
 
-# Python 2.5 compat fix
-if not hasattr(urlparse, 'parse_qsl'):
-    import cgi
-    urlparse.parse_qsl = cgi.parse_qsl
-
 
 from heatclient import exc
 
@@ -56,11 +53,12 @@ class HTTPClient(object):
         self.username = kwargs.get('username')
         self.password = kwargs.get('password')
         self.region_name = kwargs.get('region_name')
+        self.include_pass = kwargs.get('include_pass')
         self.connection_params = self.get_connection_params(endpoint, **kwargs)
 
     @staticmethod
     def get_connection_params(endpoint, **kwargs):
-        parts = urlparse.urlparse(endpoint)
+        parts = urlutils.urlparse(endpoint)
 
         _args = (parts.hostname, parts.port, parts.path)
         _kwargs = {'timeout': float(kwargs.get('timeout', 600))}
@@ -140,13 +138,15 @@ class HTTPClient(object):
             kwargs['headers'].setdefault('X-Auth-Url', self.auth_url)
         if self.region_name:
             kwargs['headers'].setdefault('X-Region-Name', self.region_name)
+        if self.include_pass and not 'X-Auth-Key' in kwargs['headers']:
+            kwargs['headers'].update(self.credentials_headers())
 
         self.log_curl_request(method, url, kwargs)
         conn = self.get_connection()
 
         try:
             conn_params = self.connection_params[1][2]
-            conn_url = os.path.normpath('%s/%s' % (conn_params, url))
+            conn_url = posixpath.normpath('%s/%s' % (conn_params, url))
             conn.request(method, conn_url, **kwargs)
             resp = conn.getresponse()
         except socket.gaierror as e:
@@ -163,7 +163,15 @@ class HTTPClient(object):
         body_str = ''.join([chunk for chunk in body_iter])
         self.log_http_response(resp, body_str)
 
-        if 400 <= resp.status < 600:
+        if not 'X-Auth-Key' in kwargs['headers'] and \
+                (resp.status == 401 or
+                (resp.status == 500 and "(HTTP 401)" in body_str)):
+            raise exc.HTTPUnauthorized("Authentication failed. Please try"
+                                       " again with option "
+                                       "--include-password or export "
+                                       "HEAT_INCLUDE_PASSWORD=1\n%s"
+                                       % body_str)
+        elif 400 <= resp.status < 600:
             raise exc.from_response(resp, body_str)
         elif resp.status in (301, 302, 305):
             # Redirected. Reissue the request to the new location.
