@@ -11,19 +11,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import cStringIO
 import httplib2
 import os
 import re
+import six
 import sys
-import urllib2
 import yaml
 
 import fixtures
-import mox
+import tempfile
 import testscenarios
 import testtools
 
+from heatclient.openstack.common.py3kcompat import urlutils
+from mox3 import mox
+
 try:
     import json
 except ImportError:
@@ -45,8 +47,14 @@ TEST_VAR_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__),
 class TestCase(testtools.TestCase):
 
     def set_fake_env(self, fake_env):
-        for key, value in fake_env.items():
-            self.useFixture(fixtures.EnvironmentVariable(key, value))
+        client_env = ('OS_USERNAME', 'OS_PASSWORD', 'OS_TENANT_ID',
+                      'OS_TENANT_NAME', 'OS_AUTH_URL', 'OS_REGION_NAME',
+                      'OS_AUTH_TOKEN', 'OS_NO_CLIENT_AUTH', 'OS_SERVICE_TYPE',
+                      'OS_ENDPOINT_TYPE', 'HEAT_URL')
+
+        for key in client_env:
+            self.useFixture(
+                fixtures.EnvironmentVariable(key, fake_env.get(key)))
 
     # required for testing with Python 2.6
     def assertRegexpMatches(self, text, expected_regexp, msg=None):
@@ -62,7 +70,7 @@ class TestCase(testtools.TestCase):
     def shell_error(self, argstr, error_match):
         orig = sys.stderr
         try:
-            sys.stderr = cStringIO.StringIO()
+            sys.stderr = six.StringIO()
             _shell = heatclient.shell.HeatShell()
             _shell.main(argstr.split())
         except Exception as e:
@@ -103,7 +111,30 @@ class EnvVarTest(TestCase):
         }
         fake_env[self.remove] = None
         self.set_fake_env(fake_env)
-        self.shell_error('list', self.err)
+        self.shell_error('stack-list', self.err)
+
+
+class EnvVarTestToken(TestCase):
+
+    scenarios = [
+        ('tenant_id', dict(
+            remove='OS_TENANT_ID',
+            err='You must provide a tenant_id')),
+        ('auth_url', dict(
+            remove='OS_AUTH_URL',
+            err='You must provide an auth url')),
+    ]
+
+    def test_missing_auth(self):
+
+        fake_env = {
+            'OS_AUTH_TOKEN': 'atoken',
+            'OS_TENANT_ID': 'tenant_id',
+            'OS_AUTH_URL': 'http://no.where',
+        }
+        fake_env[self.remove] = None
+        self.set_fake_env(fake_env)
+        self.shell_error('stack-list', self.err)
 
 
 class ShellParamValidationTest(TestCase):
@@ -177,9 +208,9 @@ class ShellValidationTest(TestCase):
             'OS_AUTH_URL': 'http://no.where',
         }
         self.set_fake_env(fake_env)
-        self.shell_error('list', failed_msg)
+        self.shell_error('stack-list', failed_msg)
 
-    def test_create_validation(self):
+    def test_stack_create_validation(self):
         self.m.StubOutWithMock(ksclient, 'Client')
         self.m.StubOutWithMock(v1client.Client, 'json_request')
         fakes.script_keystone_client()
@@ -193,36 +224,34 @@ class ShellValidationTest(TestCase):
         }
         self.set_fake_env(fake_env)
         self.shell_error(
-            'create teststack '
+            'stack-create teststack '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
             'LinuxDistribution=F17"',
             'Need to specify exactly one of')
 
 
-class ShellTest(TestCase):
+class ShellBase(TestCase):
 
-    # Patch os.environ to avoid required auth info.
     def setUp(self):
-        super(ShellTest, self).setUp()
+        super(ShellBase, self).setUp()
         self.m = mox.Mox()
         self.m.StubOutWithMock(ksclient, 'Client')
         self.m.StubOutWithMock(v1client.Client, 'json_request')
         self.m.StubOutWithMock(v1client.Client, 'raw_request')
         self.addCleanup(self.m.VerifyAll)
         self.addCleanup(self.m.UnsetStubs)
-        fake_env = {
-            'OS_USERNAME': 'username',
-            'OS_PASSWORD': 'password',
-            'OS_TENANT_NAME': 'tenant_name',
-            'OS_AUTH_URL': 'http://no.where',
-        }
-        self.set_fake_env(fake_env)
+
+        # Some tests set exc.verbose = 1, so reset on cleanup
+        def unset_exc_verbose():
+            exc.verbose = 0
+
+        self.addCleanup(unset_exc_verbose)
 
     def shell(self, argstr):
         orig = sys.stdout
         try:
-            sys.stdout = cStringIO.StringIO()
+            sys.stdout = six.StringIO()
             _shell = heatclient.shell.HeatShell()
             _shell.main(argstr.split())
         except SystemExit:
@@ -235,6 +264,12 @@ class ShellTest(TestCase):
 
         return out
 
+
+class ShellTestCommon(ShellBase):
+
+    def setUp(self):
+        super(ShellTestCommon, self).setUp()
+
     def test_help_unknown_command(self):
         self.assertRaises(exc.CommandError, self.shell, 'help foofoo')
 
@@ -266,13 +301,33 @@ class ShellTest(TestCase):
             for r in required:
                 self.assertRegexpMatches(help_text, r)
 
-    def test_list(self):
+
+class ShellTestUserPass(ShellBase):
+
+    def setUp(self):
+        super(ShellTestUserPass, self).setUp()
+        self._set_fake_env()
+
+    # Patch os.environ to avoid required auth info.
+    def _set_fake_env(self):
+        fake_env = {
+            'OS_USERNAME': 'username',
+            'OS_PASSWORD': 'password',
+            'OS_TENANT_NAME': 'tenant_name',
+            'OS_AUTH_URL': 'http://no.where',
+        }
+        self.set_fake_env(fake_env)
+
+    def _script_keystone_client(self):
         fakes.script_keystone_client()
+
+    def test_stack_list(self):
+        self._script_keystone_client()
         fakes.script_heat_list()
 
         self.m.ReplayAll()
 
-        list_text = self.shell('list')
+        list_text = self.shell('stack-list')
 
         required = [
             'id',
@@ -299,7 +354,7 @@ class ShellTest(TestCase):
             "title": "Not Found"
         }
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         fakes.script_heat_error(json.dumps(resp_dict))
 
         self.m.ReplayAll()
@@ -322,13 +377,14 @@ class ShellTest(TestCase):
             "title": "Not Found"
         }
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         fakes.script_heat_error(json.dumps(resp_dict))
 
         self.m.ReplayAll()
 
+        exc.verbose = 1
+
         try:
-            exc.verbose = 1
             self.shell("stack-show bad")
         except exc.HTTPException as e:
             expect = 'ERROR: The Stack (bad) could not be found.\n<TRACEBACK>'
@@ -336,7 +392,7 @@ class ShellTest(TestCase):
 
     def test_parsable_malformed_error(self):
         invalid_json = "ERROR: {Invalid JSON Error."
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         fakes.script_heat_error(invalid_json)
         self.m.ReplayAll()
 
@@ -356,7 +412,7 @@ class ShellTest(TestCase):
             "title": "Not Found"
         }
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         fakes.script_heat_error(json.dumps(missing_message))
         self.m.ReplayAll()
 
@@ -377,19 +433,20 @@ class ShellTest(TestCase):
             "title": "Not Found"
         }
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         fakes.script_heat_error(json.dumps(resp_dict))
         self.m.ReplayAll()
 
+        exc.verbose = 1
+
         try:
-            exc.verbose = 1
             self.shell("stack-show bad")
         except exc.HTTPException as e:
             self.assertEqual(str(e),
                              "ERROR: The Stack (bad) could not be found.\n")
 
-    def test_describe(self):
-        fakes.script_keystone_client()
+    def test_stack_show(self):
+        self._script_keystone_client()
         resp_dict = {"stack": {
             "id": "1",
             "stack_name": "teststack",
@@ -406,7 +463,7 @@ class ShellTest(TestCase):
 
         self.m.ReplayAll()
 
-        list_text = self.shell('describe teststack/1')
+        list_text = self.shell('stack-show teststack/1')
 
         required = [
             'id',
@@ -421,7 +478,7 @@ class ShellTest(TestCase):
             self.assertRegexpMatches(list_text, r)
 
     def test_template_show_cfn(self):
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         template_data = open(os.path.join(TEST_VAR_DIR,
                                           'minimal.template')).read()
         resp = fakes.FakeHTTPResponse(
@@ -448,7 +505,7 @@ class ShellTest(TestCase):
             self.assertRegexpMatches(show_text, r)
 
     def test_template_show_hot(self):
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         resp_dict = {"heat_template_version": "2013-05-23",
                      "parameters": {},
                      "resources": {},
@@ -473,8 +530,8 @@ class ShellTest(TestCase):
         for r in required:
             self.assertRegexpMatches(show_text, r)
 
-    def test_create(self):
-        fakes.script_keystone_client()
+    def test_stack_create(self):
+        self._script_keystone_client()
         resp = fakes.FakeHTTPResponse(
             201,
             'Created',
@@ -490,7 +547,7 @@ class ShellTest(TestCase):
 
         template_file = os.path.join(TEST_VAR_DIR, 'minimal.template')
         create_text = self.shell(
-            'create teststack '
+            'stack-create teststack '
             '--template-file=%s '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
@@ -506,9 +563,9 @@ class ShellTest(TestCase):
         for r in required:
             self.assertRegexpMatches(create_text, r)
 
-    def test_create_url(self):
+    def test_stack_create_url(self):
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         resp = fakes.FakeHTTPResponse(
             201,
             'Created',
@@ -523,7 +580,7 @@ class ShellTest(TestCase):
         self.m.ReplayAll()
 
         create_text = self.shell(
-            'create teststack '
+            'stack-create teststack '
             '--template-url=http://no.where/minimal.template '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
@@ -538,9 +595,9 @@ class ShellTest(TestCase):
         for r in required:
             self.assertRegexpMatches(create_text, r)
 
-    def test_create_object(self):
+    def test_stack_create_object(self):
 
-        fakes.script_keystone_client()
+        self._script_keystone_client()
         template_file = os.path.join(TEST_VAR_DIR, 'minimal.template')
         template_data = open(template_file).read()
         v1client.Client.raw_request(
@@ -563,7 +620,7 @@ class ShellTest(TestCase):
         self.m.ReplayAll()
 
         create_text = self.shell(
-            'create teststack2 '
+            'stack-create teststack2 '
             '--template-object=http://no.where/container/minimal.template '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
@@ -578,8 +635,8 @@ class ShellTest(TestCase):
         for r in required:
             self.assertRegexpMatches(create_text, r)
 
-    def test_update(self):
-        fakes.script_keystone_client()
+    def test_stack_update(self):
+        self._script_keystone_client()
         resp = fakes.FakeHTTPResponse(
             202,
             'Accepted',
@@ -596,7 +653,7 @@ class ShellTest(TestCase):
 
         template_file = os.path.join(TEST_VAR_DIR, 'minimal.template')
         create_text = self.shell(
-            'update teststack2/2 '
+            'stack-update teststack2/2 '
             '--template-file=%s '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
@@ -611,8 +668,8 @@ class ShellTest(TestCase):
         for r in required:
             self.assertRegexpMatches(create_text, r)
 
-    def test_delete(self):
-        fakes.script_keystone_client()
+    def test_stack_delete(self):
+        self._script_keystone_client()
         resp = fakes.FakeHTTPResponse(
             204,
             'No Content',
@@ -625,7 +682,7 @@ class ShellTest(TestCase):
 
         self.m.ReplayAll()
 
-        create_text = self.shell('delete teststack2/2')
+        create_text = self.shell('stack-delete teststack2/2')
 
         required = [
             'stack_name',
@@ -637,6 +694,58 @@ class ShellTest(TestCase):
             self.assertRegexpMatches(create_text, r)
 
 
+class ShellTestToken(ShellTestUserPass):
+
+    # Rerun all ShellTestUserPass test with token auth
+    def setUp(self):
+        self.token = 'a_token'
+        super(ShellTestToken, self).setUp()
+
+    def _set_fake_env(self):
+        fake_env = {
+            'OS_AUTH_TOKEN': self.token,
+            'OS_TENANT_ID': 'tenant_id',
+            'OS_AUTH_URL': 'http://no.where',
+            # Note we also set username/password, because create/update
+            # pass them even if we have a token to support storing credentials
+            # Hopefully at some point we can remove this and move to only
+            # storing trust id's in heat-engine instead..
+            'OS_USERNAME': 'username',
+            'OS_PASSWORD': 'password'
+        }
+        self.set_fake_env(fake_env)
+
+    def _script_keystone_client(self):
+        fakes.script_keystone_client(token=self.token)
+
+
+class ShellTestStandaloneToken(ShellTestUserPass):
+
+    # Rerun all ShellTestUserPass test in standalone mode, where we
+    # specify --os-no-client-auth, a token and Heat endpoint
+    def setUp(self):
+        self.token = 'a_token'
+        super(ShellTestStandaloneToken, self).setUp()
+
+    def _set_fake_env(self):
+        fake_env = {
+            'OS_AUTH_TOKEN': self.token,
+            'OS_NO_CLIENT_AUTH': 'True',
+            'HEAT_URL': 'http://no.where',
+            # Note we also set username/password, because create/update
+            # pass them even if we have a token to support storing credentials
+            # Hopefully at some point we can remove this and move to only
+            # storing trust id's in heat-engine instead..
+            'OS_USERNAME': 'username',
+            'OS_PASSWORD': 'password'
+        }
+        self.set_fake_env(fake_env)
+
+    def _script_keystone_client(self):
+        # The StanaloneMode shouldn't need any keystoneclient stubbing
+        pass
+
+
 class ShellEnvironmentTest(TestCase):
 
     def setUp(self):
@@ -646,16 +755,34 @@ class ShellEnvironmentTest(TestCase):
         self.addCleanup(self.m.VerifyAll)
         self.addCleanup(self.m.UnsetStubs)
 
-    def collect_links(self, env, content, url, map_name):
+    def collect_links(self, env, content, url, env_base_url=''):
+
         jenv = yaml.safe_load(env)
         fields = {'files': {}}
-        self.m.StubOutWithMock(urllib2, 'urlopen')
-        urllib2.urlopen(url).AndReturn(cStringIO.StringIO(content))
-        self.m.ReplayAll()
+        if url:
+            self.m.StubOutWithMock(urlutils, 'urlopen')
+            urlutils.urlopen(url).AndReturn(six.StringIO(content))
+            self.m.ReplayAll()
 
-        v1shell._get_file_contents(jenv['resource_registry'],
-                                   fields)
-        self.assertEqual(fields['files'][map_name], content)
+        v1shell._resolve_environment_urls(fields, env_base_url, jenv)
+        if url:
+            self.assertEqual(fields['files'][url], content)
+
+    def test_prepare_environment_file(self):
+        with tempfile.NamedTemporaryFile() as env_file:
+            env = '''
+            resource_registry:
+              "OS::Thingy": "file:///home/b/a.yaml"
+            '''
+            env_file.write(env)
+            env_file.flush()
+            env_url, env_dict = v1shell._prepare_environment_file(
+                env_file.name)
+            self.assertEqual(
+                {'resource_registry': {'OS::Thingy': 'file:///home/b/a.yaml'}},
+                env_dict)
+            env_dir = os.path.dirname(env_file.name)
+            self.assertEqual(env_url, 'file://%s' % env_dir)
 
     def test_global_files(self):
         a = "A's contents."
@@ -664,7 +791,7 @@ class ShellEnvironmentTest(TestCase):
         resource_registry:
           "OS::Thingy": "%s"
         ''' % url
-        self.collect_links(env, a, url, url)
+        self.collect_links(env, a, url)
 
     def test_nested_files(self):
         a = "A's contents."
@@ -675,7 +802,7 @@ class ShellEnvironmentTest(TestCase):
             freddy:
               "OS::Thingy": "%s"
         ''' % url
-        self.collect_links(env, a, url, url)
+        self.collect_links(env, a, url)
 
     def test_http_url(self):
         a = "A's contents."
@@ -684,7 +811,7 @@ class ShellEnvironmentTest(TestCase):
         resource_registry:
           "OS::Thingy": "%s"
         ''' % url
-        self.collect_links(env, a, url, url)
+        self.collect_links(env, a, url)
 
     def test_with_base_url(self):
         a = "A's contents."
@@ -696,7 +823,29 @@ class ShellEnvironmentTest(TestCase):
             server_for_me:
               "OS::Thingy": a.yaml
         '''
-        self.collect_links(env, a, url, 'a.yaml')
+        self.collect_links(env, a, url)
+
+    def test_with_built_in_provider(self):
+        a = "A's contents."
+        env = '''
+        resource_registry:
+          resources:
+            server_for_me:
+              "OS::Thingy": OS::Compute::Server
+        '''
+        self.collect_links(env, a, None)
+
+    def test_with_env_file_base_url(self):
+        a = "A's contents."
+        url = 'file:///tmp/foo/a.yaml'
+        env = '''
+        resource_registry:
+          resources:
+            server_for_me:
+              "OS::Thingy": a.yaml
+        '''
+        env_base_url = 'file:///tmp/foo'
+        self.collect_links(env, a, url, env_base_url)
 
     def test_unsupported_protocol(self):
         env = '''
