@@ -12,7 +12,6 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
 from heatclient.v1.stacks import Stack
 from heatclient.v1.stacks import StackManager
 
@@ -90,6 +89,12 @@ class StackOperationsTest(testtools.TestCase):
         stack.update()
         manager.update.assert_called_once_with('the_stack/abcd1234')
 
+    def test_create_stack(self):
+        manager = MagicMock()
+        stack = mock_stack(manager, 'the_stack', 'abcd1234')
+        stack = stack.create()
+        manager.create.assert_called_once_with('the_stack/abcd1234')
+
 
 class StackManagerNoPaginationTest(testtools.TestCase):
 
@@ -97,7 +102,7 @@ class StackManagerNoPaginationTest(testtools.TestCase):
         ('total_0', dict(total=0)),
         ('total_1', dict(total=1)),
         ('total_9', dict(total=9)),
-        ('total_1', dict(total=10)),
+        ('total_10', dict(total=10)),
         ('total_11', dict(total=11)),
         ('total_19', dict(total=19)),
         ('total_20', dict(total=20)),
@@ -251,9 +256,9 @@ class StackManagerPaginationTest(testtools.TestCase):
 
             offset = result[0]
             url = '/stacks?'
-            if offset > 0:
-                url += 'marker=abcd1234-%s&' % offset
             url += 'limit=%s' % self.page_size
+            if offset > 0:
+                url += '&marker=abcd1234-%s' % offset
             self.assertEqual(url, arg_url)
 
             def results():
